@@ -5,7 +5,7 @@
 //      Copyright (C) 2001-2002 by Rdiger Goetz
 //      Author: Rdiger Goetz <minkowsky@r-goetz.de>
 // 
-//      Time-stamp: <30-May-2002 19:35:00 goetz>
+//      Time-stamp: <20-Sep-2002 16:21:55 goetz>
 // 
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -403,8 +403,8 @@ int addDate(char *ds)
  split(s,tp,64);
 
  // tp[0] enthlt die did, die bei add Date noch ungestetz ist
- if(!checkPermission(NULL,WRITE_NEW,uname2uid(tp[1])))
-  return(false);
+//  if(!checkPermission(NULL,WRITE_NEW,uname2uid(tp[1])))
+//   return(false);
 
  d= new Date;
  fillUserInDate(&d->init,uname2uid(tp[2]),DATE_STAT_UKN,0,PERM_ALL);
@@ -421,10 +421,12 @@ int addDate(char *ds)
  d-> stopDate = atoi(tp[6]);
  d-> stopTime = atoi(tp[ 7]);
  strcpy(d->descr,trimRight(tp[ 8]));
- checkString(d->descr);
+ // checkString(d->descr);
+ replaceBraces(d->descr);
  d->tlen=strlen(tp[ 8])+1;
  strcpy(d->descr+d->tlen,trimRight(tp[9]));
- checkString(d->descr+d->tlen);
+ // checkString(d->descr+d->tlen);
+ replaceBraces(d->descr+d->tlen);
  d->room = rname2rid(tp[10]);
  d->group = gname2gid(tp[11]);
  d->flags = DATE_FLAG_PRIVAT *atoi(tp[12]) + DATE_FLAG_GROUPINTERN *atoi(tp[13]) +
@@ -436,7 +438,8 @@ int addDate(char *ds)
  d->priority        = atoi(tp[18]);
  addReminderList2Date(tp[19],d);
 
- d->comm =NULL;
+ // d->comm =NULL;
+ d->notiz =NULL;
  d->deleted =false;
  
  dateList[dateAnz]=d;
@@ -645,10 +648,12 @@ int modifyDate(int socket,char *ds)
   strcpy(tmp+v1,ed->descr + ed->tlen);
 
  strcpy(d->descr,tmp);
- checkString(d->descr);
+ // checkString(d->descr);
+ replaceBraces(d->descr);
  d->tlen=v1;
  strcpy(d->descr+d->tlen,tmp+v1);
- checkString(d->descr+d->tlen);
+ // checkString(d->descr+d->tlen);
+ replaceBraces(d->descr+d->tlen);
 
  v1 =rname2rid(tp[10]);
  if(changed(ed->room,v1))
@@ -1251,7 +1256,7 @@ void fillUserInDate(UserInDate *uid, int id,int stat,int flag,int perm)
  uid->flags=flag;
 }
 
-
+#ifdef OLD
 void addComm(int socket,char *t)
 {
  char *tp[64],text[8192],*s;
@@ -1266,7 +1271,8 @@ void addComm(int socket,char *t)
  else
   {
    sprintf(text,"%s: %s\n",tp[1],trimRight(tp[2]));
-   checkString(text);
+   //   checkString(text);
+   replaceBraces(text);
    if(dateList[did]->comm!=NULL)
     l = strlen(dateList[did]->comm);
    else
@@ -1309,7 +1315,20 @@ void getComm (int socket,char *t)
  did =atoi(t);
  send2Client(socket,dateList[did]->comm);
 }
+#else
+void addComm(int socket,char *t)\
+{
+ send2Client(socket,"Error: Command is obsolete and unsupported");
+}
+void writeComm(int did, char *t)
+{
+}
+void getComm (int socket,char *t)
+{
+ send2Client(socket,"Error: Command is obsolete and unsupported");
+}
 
+#endif
 int sendBriefDate(int socket, char *t)
 {
  char *tp[8],buffer[65536];
@@ -1411,6 +1430,7 @@ void briefDate2String(char *buffer,Date *d,int baseDate , int uid)
  int noRem=false,igno=false;
  int refDate,refTime;
  int startDate=-1,startTime=-1,stopDate=-1,stopTime=-1;
+ char shiftDateStr[1024];
 
  perm = getPermission(d,uid,d->group);
  umodi = getUserModification4Date(d->id,uid);
@@ -1434,6 +1454,7 @@ void briefDate2String(char *buffer,Date *d,int baseDate , int uid)
    startTime = d->startTime;
    stopDate  = d->stopDate;
    stopTime  = d->stopTime;
+   shiftDateStr[0]='\0';
   } else {
    ci.startDate=d->startDate;
    ci.startTime=d->startTime;
@@ -1607,9 +1628,9 @@ int sendDate(int socket, char *t,int aquire)
   {
    strcat(rList,reminder2Str4Date(&d->reminder[i],txt));
   }
- if(d->comm!=NULL)
-  comm = d->comm;
- else
+//  if(d->comm!=NULL)
+//   comm = d->comm;
+//  else
   comm = "";
 
  //               0 1   2   3  4  5  6  7    8     9    10  11 12 13 14 15 16 17 18   19  20  21 22 23    24  25
@@ -1679,101 +1700,6 @@ int UiDlist2str(UserInDate *ul, char *t,int anz)
   }
 }
 
-int split(char *txt,char **tp ,int max)
-{
- int sc=0,bc=0,qc=0;
- int i=0,anz;
- char *s,*t;
-
- s=skip2nonWS(txt);
- if(s==NULL)
-  tp[0]=txt;
- else
-  {
-   tp[i]=s;
-   i++;
-
-   for(t=s; *t!='\0'; t++)
-    {
-     // Alle "" '' {}-Blcke dedektieren
-     if(sc==0 && qc==0) // In Quotes und strings nicht nach {}-Blcken suchen
-      {
-       if(*t=='{')
-	bc++;
-       if(*t=='}')
-	bc--;
-      }
-     if(*t=='"')
-      sc =sc ^1;
-
-     if(sc==0 && qc==0 && bc==0)// In "" '' {}-Blcken  nicht nach WS suchen
-      if(*t==' ' || *t=='\t')
-       {
-	if(t==s)             // Falls t auf den Nachfolger des letzten tp zeigt (d.h. meherer WS hinter einander
-	 {                   // verschiebe den letzten tp nur , sonst lege einen neunen an
-	  tp[i-1]=t;
-	  s=t+1;
-	 }
-	else
-	 {
-	  tp[i]=t;
-	  i++;
-	  if(i==max)
-	   break;
-	  s=t+1;
-	 }
-       }	
-    }
-  }
-
- anz=i;
- for(i=0;i<anz;i++)
-  {
-   s=skip2nonWSnl(tp[i]);
-   if(s==NULL)
-    anz=i;
-   else
-    {
-     tp[i]=s;
-     s--;
-     if(s>=txt)
-      *s='\0';
-    }
-  }
- for(i=0;i<anz;i++)
-  {
-   s=tp[i];
-   if(*s=='{')
-    {
-     tp[i]=s+1;
-     s=strrchr(tp[i],'}');
-     *s='\0';
-    }
-   if(*s=='"')
-    {
-     tp[i]=s+1;
-     s=strrchr(tp[i],'"');
-     *s='\0';
-    }
-   if(*s=='\'')
-    {
-     tp[i]=s+1;
-     s=strrchr(tp[i],'\'');
-     *s='\0';
-    }
-  }
-
-
- if (anz >0)
-  {
-   s=tp[anz-1]+strlen(tp[anz-1])-1;
-   while (*s== ' ' || *s =='\t')
-    s--;
-   s++;
-   *s='\0';
-  }
- return(anz);
-}
 
 void exportDate(Date *d,char *file)
 {
@@ -1930,7 +1856,7 @@ int writeDates()
   {
    printf("dirty shifts\n");
    sprintf(file,"%s/dates.shift",dataDir);
-   fp=fopen(file,"a");
+   fp=fopen(file,"r+");
    if(fp==NULL)
     {
      printf("can't open %s\n",file);
@@ -2171,10 +2097,14 @@ int convertDateFrom0400(Date *d, Date_0400 *dold)
   d->stopTime        = dold->stopTime;
   memcpy(d->descr,dold->descr,768);
   d->descr[767]      ='\0';
+#ifdef OLD
   writeComm(d->id,dold->comm);
   t = new char [strlen(dold->comm)+4];
   d->comm=t;
   strcpy(d->comm,dold->comm);
+#else
+  d->notiz = NULL;
+#endif
   d->dateShiftAnz    = dold->dateShiftAnz;
   d->dateShiftMax    = dold->dateShiftMax;
   d->dateShiftPtr    = dold->dateShiftPtr;
@@ -2313,28 +2243,16 @@ int readDates()
     }
   }
 
-//  maxDid=dateAnz;
-//  for(i=0;i<dateAnz; i++)
-//   {
-//    int did;
-//    did = d[i].id;
-//    if(did>dateMax)
-//     extendDateList();
-//    if(did>maxDid)
-//      maxDid=did;
-//    dateList[did] = d +i; 
-//    d[i].dirty=false;
-//   }
  dateCount=dateAnz;
  dateAnz =maxDid+1;
 
  // sortDateList(dateList,dateAnz);
  printf("\t\t\t\t %d date read\n",dateAnz);
 
-#ifdef V0506
  for(i=0;i<dateAnz; i++)
   if(dateList[i] != NULL)
    {
+    dateList[i]->notiz = NULL;
     // Um alte Daten einzulesen
     if(dateList[i]->repeatTyp != NO_REPEAT)
      {
@@ -2368,43 +2286,8 @@ int readDates()
     recalcReminder(dateList[i]);
     initDateShiftList(dateList[i]);
    }
-#else
- for(i=0;i<dateCount; i++)
-  {
-   // Um alte Daten einzulesen
-   if(d[i].repeatTyp != NO_REPEAT)
-    {
-     if(d[i].repeatStopDate<1900000 && d[i].repeatStopDate  != 0)
-      {
-       printf("INFO: RepeatCount of %d ...",d[i].repeatStopDate);
-       if(inputFormatVersion==400)
-	d[i].repeatStopDate = skip2Last(d+i);
-       else
-	d[i].repeatStopDate =0;
-       printf("is converted to repeatStopDate %d\n",d[i].repeatStopDate);
-      }
-    }
 
-   for(j=0;j<d[i].partAnz; j++)
-    addDate2Calendar(d+i,d[i].part[j].id);
-   addDate2RoomCalendar(d+i,d[i].room);
-   
-   for(r=0;r<REMINDERS_IN_DATE ; r++)
-    {
-     if(r<d[i].remAnz)
-      {
-       int lst = d[i].reminder[r].ct.tim;
-       d[i].reminder[r].ct.ctrl = new ReminderControl;
-       d[i].reminder[r].ct.ctrl->lastSendTim = lst;
-      }
-     else
-      d[i].reminder[r].ct.ctrl=NULL;
-    }
-   
-   recalcReminder(d+i);
-   initDateShiftList(d+i);
-  }
-#endif
+
  printf("reading shifts\n");
  readDateShifts();
 
@@ -2502,6 +2385,7 @@ int readDateShifts()
  Date *d;
  FileHeader fh;
  char *buffer;
+ int shiftsDirty=false;
 
 
  sprintf(file,"%s/dates.shift",dataDir);
@@ -2595,9 +2479,43 @@ int readDateShifts()
    d= dateList[i];
    if(d!=NULL)
     if(d->dateShiftAnz>0)
-     
-     qsort(d->shifts,d->dateShiftAnz,sizeof(DateShift *),compareDateShiftsByDate);
+     {
+      int dirty=true;
+      qsort(d->shifts,d->dateShiftAnz,sizeof(DateShift *),compareDateShiftsByDate);
+      
+      for(j=1; j< d->dateShiftAnz ; j++)
+       {
+	if(d->shifts[j]->refDate == d->shifts[j-1]->refDate)
+	 if(d->shifts[j]->refTime == d->shifts[j-1]->refTime)
+	  {
+	   printf("Shifts #%d and #%d on date %d '%s' refer to some event. removing shift #%d\n",
+		  j-1,j,d,d->descr,j-1);
+	   minkoLog("Shifts #%d and #%d on date %d '%s' refer to some event. removing shift #%d\n",
+		  j-1,j,d,d->descr,j-1);
+	   d->shifts[j-1]->valid =false;
+	   d->shifts[j]->dirty =true; // force writeShifts
+	   dirty =true;
+	  }
+       }
+      
+      if(dirty)
+       {
+	qsort(d->shifts,d->dateShiftAnz,sizeof(DateShift *),compareDateShiftsByDate);
+	for(j=0; j< d->dateShiftAnz ; j++)
+	 if(!d->shifts[j]->valid)
+	  {
+	   printf("Shorten shoft list of date %s '%s' from %d entries to %d\n",d,d->descr,d->dateShiftAnz,j);
+	   minkoLog("Shorten shoft list of date %s '%s' from %d entries to %d\n",d,d->descr,d->dateShiftAnz,j);
+	   d->dateShiftAnz =j;
+	   break;
+	  }
+	shiftsDirty=true;
+       }
+     }
   }
+ if(shiftsDirty)
+  writeAllDateShifts(false);
+
 }
 
 void addDate2Calendar(Date*d, int id)
@@ -2791,7 +2709,8 @@ DateShift *getDateShift4Date(Date *d, int date, int stime,int create)
  for(i=0;i< d->dateShiftAnz; i++)
   if(d->shifts[i]->refDate ==date)
    if(d->shifts[i]->refTime ==stime)
-    return(d->shifts[i]);
+    if(d->shifts[i]->valid)
+     return(d->shifts[i]);
  
  if(create)
   {
@@ -2846,7 +2765,9 @@ void checkDateShiftsAndRecontruct()
      dateShiftList[i]->valid = false;
    }
  for(i=0;i<dateAnz; i++)
-  dateList[i]->dateShiftAnz=0;
+  if(dateList[i]!=NULL)
+   if(! dateList[i]->deleted)
+    dateList[i]->dateShiftAnz=0;
 
  // Baue shift-Listen an den Date neu auf.
  for(i=0;i<dateShiftAnz; i++)
@@ -2870,6 +2791,15 @@ int compareDateShiftsByDate(const void *v1, const void * v2)
 
  dsh1 = *((DateShift **) v1) ;
  dsh2 = *((DateShift **) v2) ;
+
+ // invalid shift must be sort to the end
+ if(!dsh1->valid &&  dsh2->valid)
+  return(1);
+ if( dsh1->valid && !dsh2->valid)
+  return(-1);
+ if(!dsh1->valid && !dsh2->valid)
+  return(0);
+
  if( dsh1->startDate < dsh2->startDate)
   return(-1);
  else if ( dsh1->startDate > dsh2->startDate)
@@ -3217,7 +3147,7 @@ void getUserCal(char *str,int socket)
  char buffer[250000],tx[1024];
 
  split(str,tp,8);
-  uid = uname2uid(tp[0]);
+ uid = uname2uid(tp[0]);
  start = atoi(tp[2]);
  stop  = atoi(tp[3]);
  igno  = atoi(tp[4]);
@@ -3266,9 +3196,9 @@ void getUserCal(char *str,int socket)
       }
      else if (list[i][0] == 'g')
       {
-       cid = uname2uid(trimRight(nam));
+       cid = gname2gid(trimRight(nam));
        if(cid>=0)
-	addCal2Buffer(buffer,&groupList[cid].cal,uid,cid,start,stop,igno,ignoInfo,stamp);
+	addCal2Buffer(buffer,&groupList[cid].cal,uid,cid+GROUPOFFSET,start,stop,igno,ignoInfo,stamp);
       }
      else if (list[i][0] == 'u')
       {
@@ -3912,12 +3842,12 @@ void addCal2Buffer(char *buffer, Calendar *cal, int uid,int cid,int startDay, in
    if (cal->list[i]->internalStatus !=stamp)
     { //stamp
      dateAsInfo = dateOnlyAsInfo(cal->list[i],uid);
-     if(dateAsInfo)
-      printf("date '%s' only as info for %d\n",cal->list[i]->descr,uid);
+//      if(dateAsInfo)
+//       printf("date '%s' only as info for %d\n",cal->list[i]->descr,uid);
      if(ignoInfo == 1)
       if(dateAsInfo)
        {
-	printf("date is ignored for %d (%d) %d\n",uid,ignoInfo,dateAsInfo);
+// 	printf("date is ignored for %d (%d) %d\n",uid,ignoInfo,dateAsInfo);
 	continue;
        }
      
@@ -4067,7 +3997,7 @@ void getRoomCal(char *buffer, int uid, int cid, int startDay, int stopDay,int ig
      if(ignoInfo == 1)
       if(dateAsInfo)
        continue;
-     perm = getPermission(roomList[cid].cal.list[i],uid,cid);
+     perm = getPermission(roomList[cid].cal.list[i],uid,cid+ROOMOFFSET);
      if( (perm & READ_TIMES) == READ_TIMES )
       {
        ci.startDate=roomList[cid].cal.list[i]->startDate;
@@ -4188,15 +4118,54 @@ int dateOnlyAsInfo(Date *d,int uid)
 
 
 
-void checkString(char *t)
-{
- char *s;
 
- for(s=t; *s!='\0'; s++)
+
+
+int readNotiz(Date *d)
+{
+ FILE *fp;
+ char file[1024];
+ struct stat fstat;
+
+ if(d->notiz == NULL)
+  d->notiz = initNotizList(d->id,CALENDAR_NOTIZ);
+
+ sprintf(file,"%s/notiz/Cal%08d.txt",dataDir,d->id);
+ if(stat(file,&fstat)<0)
+  return(false);
+ 
+ fp=fopen(file,"r");
+ if(fp==NULL)
   {
-   if( *s=='{')
-    *s='(';
-   if( *s=='}')
-    *s='}';
+   printf("can't open %s\n",file);
+   return(false);
   }
+
+ printf("reading Notiz from '%s'\n",file);
+ readNotiz(fp,d->notiz);
+ fclose(fp);
+ return(true); 
+}
+
+int writeNotiz(Date *d)
+{
+ char file[1024];
+ FILE *fp;
+
+ if(d->notiz == NULL)
+  return(true);
+ 
+ printf("writing Notiz : %p\n",d->notiz);
+ 
+ sprintf(file,"%s/notiz/Cal%08d.txt",dataDir,d->id);
+ fp=fopen(file,"w");
+ if(fp==NULL)
+  {
+   printf("can't open %s\n",file);
+   delete d->notiz;
+   d->notiz=NULL;
+   return(false);
+  }
+ writeNotiz(fp,d->notiz);
+ fclose(fp);
 }
