@@ -5,7 +5,7 @@
 //      Copyright (C) 2001-2002 by Rdiger Goetz
 //      Author: Rdiger Goetz <minkowsky@r-goetz.de>
 // 
-//      Time-stamp: <01-May-2002 14:48:12 goetz>
+//      Time-stamp: <14-Oct-2002 19:54:23 goetz>
 // 
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -36,8 +36,6 @@
 
 void extendTodoList(TodoList *todos);
 void extendProjList(ProjectList *projs);
-int tid2idx(int tid);
-int pid2idx(int pid);
 int fillStr2Todo(char *tp[],TodoItem *todo);
 void addTODO2Lists(TodoItem *todo);
 void addTODO2GroupsList(TodoItem *todo, int gid);
@@ -46,14 +44,15 @@ int fillStr2Project(char *tp[],Project *proj);
 void addProject2Lists(Project *proj);
 void addProject2GroupsList(Project *proj, int gid);
 void addProject2UsersList(Project *proj, int uid);
-int  permission4Proj(Project *proj, int uid);
-int  permission4Todo(TodoItem *todo, int uid);
 void addTODO2str(char *t, TodoItem *todo, int uid,int enclose);
 void addProj2str(char *t, Project *proj, int uid,int enclose);
 int deletePermission4Todo(TodoItem *todo,int uid);
 int deletePermission4Proj(Project *proj,int uid);
 TodoItem * readTodoList_0400(FILE *fp,int fsize);
 extern int convertReminderFrom0400(Reminder *rem, Reminder_0400 *old);
+int isProjectIncluded(int pid1, int pid2);
+void getSubProjects();
+void getSubProjects( int pidx);
 
 FileHeader todoFileHeader,projFileHeader;
 
@@ -80,7 +79,7 @@ void extendTodoList()
  int anz,anz0;
  int i;
  
- printf("extending space for addresses\n");
+ // printf("extending space for addresses\n");
 
  todoBlocks++;
  anz= todoBlocks * DATEBLOCKSIZE;
@@ -130,7 +129,7 @@ void extendProjList()
  int anz,anz0;
  int i;
  
- printf("extending space for addresses\n");
+ // printf("extending space for addresses\n");
 
  projBlocks++;
  anz= projBlocks * DATEBLOCKSIZE;
@@ -256,6 +255,7 @@ int readTodoList()
 
    if(todoList[i]->id>lastTid)
     lastTid = todoList[i]->id;
+   todoList[i]->notiz =NULL;
    addTODO2Lists(todo+i);
   }
  if(modified)
@@ -444,12 +444,27 @@ int readProjList()
     }
   }
 
- lastPid=-1;
+ int shiftProjectIDs=false;
+
  for(i=0;i<projAnz; i++)
   {
    projList[i] = proj+i;
+   if(projList[i]->id == 0 )
+    shiftProjectIDs =true;
+  }
+
+ if(shiftProjectIDs)
+  printf("Warning: Project -IDs are shifted by 1 to avoid project0ID==0\n");
+
+ lastPid=-1;
+ for(i=0;i<projAnz; i++)
+  {
+   if(shiftProjectIDs)
+    projList[i]->id++;
+
    if(projList[i]->id>lastPid)
     lastPid = projList[i]->id;
+   projList[i]->notiz = NULL;
    addProject2Lists(proj+i);
   }
  if(modified)
@@ -798,6 +813,8 @@ int changeProject(int socket,char *ds)
  writeProjList();
 }
 
+
+
 //     #Project-Item
 //     # ID type perm open/close status Titel start stop {users ... } init dringl. -dynamisch {todo-List} 
 //     # 0   1     2        3       4     5     6     7       8         9     10         11         12
@@ -868,11 +885,19 @@ int fillStr2Project(char *tp[],Project *proj)
 //     # Depend-List: jedes Depend: {tid type val}
  anz = split(tp[ 8],tp2,130);
  stat = TODO_STAT_DONE;
+ int tanz=0;
  for(i=0; i<anz; i++)
   {
-   int idx;
+   int idx,tid;
    split(tp2[i],tp3,3);
-   proj->todo[i].tid = atoi(tp3[0]);
+   tid = atoi(tp3[0]);
+
+   // Check if proj is already subproject to project #-tid
+   if (tid<0)
+    if(isProjectIncluded(proj->id,-tid))
+     continue;
+
+   proj->todo[i].tid = tid;
    proj->todo[i].lvl = atoi(tp3[2]);
    trimRight(tp3[1]);
    if (tp3[1][0] != '\0')
@@ -889,16 +914,69 @@ int fillStr2Project(char *tp[],Project *proj)
     }
    else
     proj->todo[i].anz=0;
-   idx = tid2idx(proj->todo[i].tid);
-   if(stat< todoList[idx]->status)
-    stat = todoList[idx]->status;
+   
+   if(tid >=0 )
+    {
+     idx = tid2idx(tid);
+     if(stat< todoList[idx]->status)
+      stat = todoList[idx]->status;
+    } else { 
+     idx = pid2idx(-tid);
+     if(stat< projList[idx]->status)
+      stat = projList[idx]->status;
+    }
+   tanz++;
   }
  proj->status =stat;
- proj->tanz=anz;
+ proj->tanz=tanz;
 
 
 }
 
+
+// isProjectIncluded check if project #pid1 is subproject to project #pid2
+int isProjectIncluded(int pid1, int pid2)
+{
+ Project *proj;
+
+ proj =  projList[pid2idx(pid2)];
+ for(int i=0 ;i<proj->tanz; i++)
+  if(proj->todo[i].tid<0)
+   {
+    if(proj->todo[i].tid == -pid1)
+     return true;
+    else
+     return isProjectIncluded(pid1,-proj->todo[i].tid);
+   }
+ return false;
+}
+void getSubProjects()
+{
+ int i;
+
+ for(i=0;i<projAnz ; i++)
+  projList[i]->isSubProject == 0;
+ 
+ for(i=0;i<projAnz ; i++)
+  if(projList[i]->isSubProject==0)
+   getSubProjects(i);
+}
+
+void getSubProjects( int pidx)
+{
+ Project *proj;
+ int idx;
+
+ proj =  projList[pidx];
+ for(int i=0 ;i<proj->tanz; i++)
+  if(proj->todo[i].tid<0)
+   {
+    idx= pid2idx(-proj->todo[i].tid);
+    projList[idx]->isSubProject=1;
+    getSubProjects(idx);
+   }
+}
+
 void addProject2Lists(Project *proj)
 {
  int i;
@@ -1086,6 +1164,8 @@ int sendProjectAndTodoLists(int socket,char *str)
 
  anz = split(tp[1],tp2,1023);
 
+
+ getSubProjects();
  for(i=0;i<projAnz;i++)
   projList[i]->tmp = TODO_SEND_NOT;
  for (i=0;i<todoAnz; i++)
@@ -1390,15 +1470,15 @@ void addProj2str(char *t, Project *proj,int uid,int enclose)
    strcpy(uperm,"rwd");
 
  if(enclose)
-  sprintf(buffer,"{%d %s %s {%s} %d %d {%s} %s {%s} {%s} {%s} %s} ",
+  sprintf(buffer,"{%d %s %s {%s} %d %d {%s} %s {%s} {%s} {%s} %s %d} ",
 	  proj->id,uperm,openStr,replaceBraces(proj->descr)
 	  ,0,proj->stopDate,uList,userList[proj->init].name,
-	  tList,rList,replaceBraces(proj->descr+proj->tlen),gname);
+	  tList,rList,replaceBraces(proj->descr+proj->tlen),gname,proj->isSubProject);
  else
-  sprintf(buffer,"%d %s %s {%s} %d %d {%s} %s {%s} {%s} {%s} %s ",
+  sprintf(buffer,"%d %s %s {%s} %d %d {%s} %s {%s} {%s} {%s} %s %d",
 	  proj->id,uperm,openStr,replaceBraces(proj->descr),0,
 	  proj->stopDate,uList,userList[proj->init].name,
-	  tList,rList,replaceBraces(proj->descr+proj->tlen),gname);
+	  tList,rList,replaceBraces(proj->descr+proj->tlen),gname,proj->isSubProject);
  strcat(t,buffer);
 }
 
@@ -1410,6 +1490,106 @@ int getLastProjTodoChange(int socket)
  return true;
 }
 
+
+int readNotiz(TodoItem *todo)
+{
+ FILE *fp;
+ char file[1024];
+ struct stat fstat;
+
+ if(todo->notiz == NULL)
+  todo->notiz = initNotizList(todo->id,TASK_NOTIZ);
+
+ sprintf(file,"%s/notiz/Task%08d.txt",dataDir,todo->id);
+ if(stat(file,&fstat)<0)
+  return(false);
+ 
+ fp=fopen(file,"r");
+ if(fp==NULL)
+  {
+   printf("can't open %s\n",file);
+   return(false);
+  }
+
+ // printf("reading Notiz form '%s'\n",file);
+ readNotiz(fp,todo->notiz);
+ fclose(fp);
+ return(true); 
+}
+int readNotiz(Project *proj)
+{
+ FILE *fp;
+ char file[1024];
+ struct stat fstat;
+
+ if(proj->notiz == NULL)
+  {
+   //   printf("\t\tread: initNotizList:\n");
+   proj->notiz = initNotizList(proj->id,PROJECT_NOTIZ);
+   //   printf("\t\tread: initNotizList: -> %p\n",proj->notiz);
+  }
+
+ sprintf(file,"%s/notiz/Proj%08d.txt",dataDir,proj->id);
+ if(stat(file,&fstat)<0)
+  return(false);
+ 
+ fp=fopen(file,"r");
+ if(fp==NULL)
+  {
+   printf("can't open %s\n",file);
+   return(false);
+  }
+
+ // printf("reading Notiz form '%s'\n",file);
+ readNotiz(fp,proj->notiz);
+ fclose(fp);
+ return(true); 
+}
+
+int writeNotiz(TodoItem *todo)
+{
+ char file[1024];
+ FILE *fp;
+
+ if(todo->notiz == NULL)
+  return(true);
+ printf("writing Notiz : %p\n",todo->notiz);
+ 
+ sprintf(file,"%s/notiz/Task%08d.txt",dataDir,todo->id);
+ fp=fopen(file,"w");
+ if(fp==NULL)
+  {
+   printf("can't open %s\n",file);
+   delete todo->notiz;
+   todo->notiz=NULL;
+   return(false);
+  }
+ writeNotiz(fp,todo->notiz);
+ fclose(fp);
+}
+int writeNotiz(Project *proj)
+{
+ char file[1024];
+ FILE *fp;
+
+ if(proj->notiz == NULL)
+  return(true);
+
+ printf("writing Notiz : %p\n",proj->notiz);
+ 
+ sprintf(file,"%s/notiz/Proj%08d.txt",dataDir,proj->id);
+ fp=fopen(file,"w");
+ if(fp==NULL)
+  {
+   printf("can't open %s\n",file);
+   delete proj->notiz;
+   proj->notiz=NULL;
+   return(false);
+  }
+ writeNotiz(fp,proj->notiz);
+ fclose(fp);
+}
+
 // Utis
 
 int tid2idx(int tid)
@@ -1429,16 +1609,5 @@ int pid2idx(int pid)
  return(-1);
 }
 
-char *replaceBraces(char *t)
-{
- char *s;
 
- for(s=t; *s!='\0'; s++)
-  {
-   if(*s=='{')
-    *s='(';
-   else if (*s=='}')
-    *s=')';
-  }
- return(t);
-}
+
