@@ -5,7 +5,7 @@
 //      Copyright (C) 2001-2002 by Rdiger Goetz
 //      Author: Rdiger Goetz <minkowsky@r-goetz.de>
 // 
-//      Time-stamp: <28-May-2002 12:52:08 goetz>
+//      Time-stamp: <15-Sep-2002 12:25:26 goetz>
 // 
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -730,356 +730,6 @@ int compReminder(const void * p1,const void *p2)
  return(t1-t2);
 }
 
-char *date2Str(int datum, int tim,char *t)
-{
- int day,mon,year,hour,min;
- 
- year = datum /10000;
- mon  = (datum%10000) /100;
- day  = datum %100;
- hour = tim /100;
- min  = tim %100;
- 
- sprintf(t,"%2d.%2d.%4d, %2d:%02d Uhr",day,mon,year,hour,min);
- return(t);
-}
-char *date2Str(int datum, char *t)
-{
- int day,mon,year,hour,min;
- 
- year = datum /10000;
- mon  = (datum%10000) /100;
- day  = datum %100;
- 
- sprintf(t,"%2d.%2d.%4d",day,mon,year);
- return(t);
-}
-char *time2Str(int tim,char *t)
-{
- int day,mon,year,hour,min;
- 
- hour = tim /100;
- min  = tim %100;
- 
- sprintf(t,"%2d:%02d Uhr",hour,min);
- return(t);
-}
-
-int weekDayByDate(int datum)
-{
- struct tm tm,*tmp;
- int tim;
-
- tm.tm_mday = datum %100;
- tm.tm_mon  = (datum%10000)/100 -1;
- tm.tm_year = datum /10000-1900;
- tm.tm_hour = 12;
- tm.tm_min  =tm.tm_sec =0;
- tmp=&tm;
- tim = mktime(&tm);
- tmp =localtime((const time_t *)&tim);
- return((tmp->tm_wday+6) %7);
-}
-
-int calcDate (int *datum, int *tim, int offset, int offset2 , int unit)
-{
- int day,mon,year,hour,min;
- int monLen[13]={0,31,28,31,30,31,30,31,31,30,31,30,31},feb;
- struct tm tms;
-
- // printf("shift %d, %d  by %d/%d %c to \t",*datum,*tim,offset,offset2,unit);
-
- year = *datum /10000;
- mon  = (*datum%10000) /100;
- day  = *datum %100;
- hour = *tim /100;
- min  = *tim %100;
- 
- switch (unit)
-  {
-  case 'm': 
-   min  += offset; 
-   break;
-  case 'h': 
-   hour += offset;
-   min  += offset2;
-   break;
-  case 'd':
-   day  += offset;
-   hour += offset2;
-   break;
-  case 'w' :
-   day  += offset*7 + offset2;
-   break;
-  case 'M' :
-   mon  += offset;
-   day  += offset2;
-   break;
-  case 'y' :
-   year += offset;
-   mon  += offset2;
-   break;
-
-  default:
-   return(false);
-  }
-
-
-
- while(min < 0)
-  {
-   min+=60;
-   hour  --;
-  }
- while(min>60)
-  {
-   min-=60;
-   hour++;
-  }
- while(hour<0)
-  {
-   hour += 24;
-   day --;
-  }
- while(hour >=24)
-  {
-   hour -=24;
-   day++;
-  }
- while(day<0)
-  {
-   mon--;
-   if(mon<0)
-    {
-     mon=12;
-     year--;
-    }
-   day += monLen[mon];
-   if(mon ==2)
-    if( (year %4) == 0)
-     day ++;
-  }
- 
-
- while (mon>12)
-  {
-   mon-=12;
-   year++;
-  }
-
- if(mon==2 && (year %4)==0)
-  feb=1;
- else
-  feb=0;
-
- while(day>monLen[mon]+feb)
-  {
-   day -= (monLen[mon]+feb);
-   mon++;
-   if(mon==2 && (year %4)==0)
-    feb=1;
-   else
-    feb=0;
-   
-   if(mon>12)
-    {
-     year++;
-     mon=1;
-    }
-  }
-
- *datum = year*10000+mon*100+day;
- *tim   = 100*hour+min;
- return(true);
-}
-
-int getWeekDayInMonth(int *weekCount, int *wday,int datum)
-{
- struct tm tms,*tmp;
- int tim,day;
-
- tms.tm_sec =  0;
- tms.tm_min =  0;
- tms.tm_hour= 12;
- tms.tm_mday= day = datum %100;
- tms.tm_mon = (datum%10000) /100 -1;
- tms.tm_year= datum /10000-1900;
-
- tim = mktime(&tms);
- tmp = localtime((const time_t*) &tim);
- 
- *wday = (tmp->tm_wday+6)%7;
- *weekCount = (int)floor( (day-0.5)/7.);
-}
-
-int getCountedWeekDayDatum(int weekCount, int wday, int datum)
-{
- struct tm tms,*tmp;
- int dw,w1day,day,tim;
-
- tms.tm_sec =  0;
- tms.tm_min =  0;
- tms.tm_hour= 12;
- tms.tm_mday= 1;
- tms.tm_mon = (datum%10000) /100 -1;
- tms.tm_year= datum /10000-1900;
- tim = mktime(&tms);
- tmp = localtime((const time_t*) &tim);
-
- w1day = (tmp->tm_wday+6) % 7;
- dw    = (wday -w1day +7) % 7;
- day   = dw + weekCount*7 + 1;   // Die 1 , da der Refere4bnz tag der 1. des Monats ist
- datum = datum -datum %100 + day;
- return(datum);
-}
-
-int calcDateBackward (int datum, int tim, int offset , int offset2, int unit)
-{
- int day,mon,year,hour,min;
- int monLen[13]={0,31,28,31,30,31,30,31,31,30,31,30,31};
- struct tm tms;
- time_t ntim;
-
- year = datum /10000;
- mon  = (datum%10000) /100;
- day  = datum %100;
- hour = tim /100;
- min  = tim %100;
-
-//  if(verbose)
-//   printf("calcBack: %2d.%2d.%4d %2d:%02d  by %d %d %c\t",day,mon,year,hour,min,offset,offset2,unit);
-
- switch (unit)
-  {
-  case 'm': 
-   min  -= offset; 
-   break;
-  case 'h': 
-   hour -= offset;
-   min  -= offset2;
-   break;
-  case 'd':
-   day  -= offset;
-   hour -= offset2;
-   break;
-  default:
-   return(0);
-  }
-
- while(min < 0)
-  {
-   min+=60;
-   hour  --;
-  }
- while(hour<0)
-  {
-   hour += 24;
-   day --;
-  }
- while(mon<1)
-  {
-   mon+=12;
-   year --;
-  }
- while(day<0)
-  {
-   mon--;
-   if(mon<0)
-    {
-     mon=12;
-     year--;
-    }
-   day += monLen[mon];
-   if(mon ==2)
-    if( (year %4) == 0)
-     day ++;
-  }
-
-//  if(verbose)
-//   printf("-> %2d.%2d.%4d %2d:%02d  \t",day,mon,year,hour,min);
-
- tms.tm_sec =  0;
- tms.tm_min =min;
- tms.tm_hour=hour;
- tms.tm_mday=day;
- tms.tm_mon =mon-1;
- tms.tm_year=year-1900;
-
- ntim =mktime(&tms);
-//  if(verbose)
-//   printf("-> %d\n",ntim);
- 
- return(ntim);
-}
-int calcDateForward (int datum, int tim, int offset , int offset2, int unit)
-{
- int day,mon,year,hour,min;
- int monLen[13]={0,31,28,31,30,31,30,31,31,30,31,30,31};
- struct tm tms;
-
- year = datum /10000;
- mon  = (datum%10000) /100;
- day  = datum %100;
- hour = tim /100;
- min  = tim %100;
-
- switch (unit)
-  {
-  case 'm': 
-   min  += offset; 
-   break;
-  case 'h': 
-   hour += offset;
-   min  += offset2;
-   break;
-  case 'd':
-   day  += offset;
-   hour += offset2;
-   break;
-  default:
-   return(0);
-  }
-
- while(min >60)
-  {
-   min-=60;
-   hour++;
-  }
- while(hour>23)
-  {
-   hour -= 24;
-   day ++;
-  }
- while(mon>12)
-  {
-   mon-=12;
-   year++;
-  }
- while(day>monLen[mon])
-  {
-   day -= monLen[mon];
-   if(mon ==2)
-    if( (year %4) == 0)
-     day --;
-   mon++;
-   if(mon>12)
-    {
-     mon=1;
-     year++;
-    }
-  }
-
- tms.tm_sec =  0;
- tms.tm_min =min;
- tms.tm_hour=hour;
- tms.tm_mday=day;
- tms.tm_mon =mon-1;
- tms.tm_year=year-1900;
-
- 
- return(mktime(&tms));
-}
-
 int check4Reminder()
 {
  int r,ctim;
@@ -1767,12 +1417,3 @@ char * formatMail(char *mask,char *text,Reminder *r,int ext)
   }
  // printf("Formated Mail Text:\n'%s'\n",text);
 }
-int strChopLen(const char *t)
-{
- char *s;
- if(strlen(t)==0)
-  return (0);
- s = new char [strlen(t)+2];
- strcpy(s,t);
- return(strlen(strChop(s)));
-}
