@@ -29,4 +29,8 @@ if [ ! -d $4 ]
 then
     mkdirhier $4
 fi
-chown $3 $4
\ No newline at end of file
+if [ ! -d $5 ]
+then
+    mkdirhier $5
+fi
+chown $3 $4 $5
