@@ -5,7 +5,7 @@
 //      Copyright (C) 2001-2002 by Rdiger Goetz
 //      Author: Rdiger Goetz <minkowsky@r-goetz.de>
 // 
-//      Time-stamp: <07-Jun-2002 09:15:27 goetz>
+//      Time-stamp: <26-Oct-2002 15:04:45 goetz>
 // 
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,18 +25,22 @@
 /* #define V037 */
 /* #define V0372 */
 /* #define V038 */
-#define V0405
-#define V405
-#define V0409
-#define V409
-#define V0409a
-#define V0501
-#define V0505
+#include <pthread.h> 
 #define V0506
+#define WITH_CLASS
+#define WITH_THREADS
 
+
+
+#ifdef WITH_THREADS
+#include <pthread.h> 
+#endif
+
+#include "unsplitCsv.h"
 #include "perm.h"
 #include "address.h"
 
+
 #define MINKO_LANG_EN 0
 #define MINKO_LANG_DE 1
 
@@ -61,6 +65,9 @@
 #define REMIND_ALL_EXTERN 80000002
 
 
+#define MSG_MODE_CMD   0
+#define MSG_MODE_LOGIN 1
+
 #define DATEBLOCKSIZE 1024
 
 #define DATE_FLAG_PRIVAT 0x0001
@@ -132,7 +139,7 @@
 #define CURRENT_FILEVERSION_SHIFT "0410"
 #define CURRENT_FILEVERSION_TODO  "0410"
 #define CURRENT_FILEVERSION_PROJ  "0410"
-#define CURRENT_FILEVERSION_ADDR  "0411"
+#define CURRENT_FILEVERSION_ADDR  "0500"
 #define CURRENT_FILEVERSION_ADDF  "0412"
 
 #define NOTIFIER_DATE_IS_ADD   1
@@ -141,7 +148,33 @@
 #define NOTIFIER_DATE_IS_SHIFT 4
 #define NOTIFIER_DATE_IS_SUSP  5
 
+#define SYNC_OFF     0
+#define SYNC_CAL     0x10
+#define SYNC_PRJ     0x20
+#define SYNC_ADR     0x40
 
+#define SYNC_MODE_IN    1
+#define SYNC_MODE_OUT   2
+#define SYNC_MODE_INOUT 3
+
+
+
+#define SYNC_STAT_DEFAULT   0 // status of all entries before sync-in,
+                              //    all entries with this status after sync-in are new to the pda
+#define SYNC_STAT_IMPORT    1 // entry has been import into Minkowsky during sync
+#define SYNC_STAT_MODI_OUT  2 // entry has been detected to be modified on Minkowsky, ready for sync out
+#define SYNC_STAT_MODI_IN   3 // entry has been detected to be modified on PDA and synced into Minkowsky
+#define SYNC_STAT_SAME      4 // entry has been detected to be identical on PDA and Minkowsky
+#define SYNC_STAT_NEW       5 // entry has been detected to be new in Minkowsky
+#define SYNC_STAT_DIFFER    6 // entry has been detected to be differ between Minkowsky and PDA, check for MODI_IN/OUT
+
+#define SYNC_FMT_FILE 1
+#define SYNC_FMT_STR  2
+
+#define SERVER_MSG_ALREADY_LOGED_IN 1
+
+struct syncUserInfoStruct;
+typedef struct  syncUserInfoStruct syncUserInfo;
 
 
 
@@ -220,7 +253,7 @@ typedef struct
  int  stopDate, stopTime;
  char addDetails [100];     // zustzliche Details
  Reminder reminder;
- short int valid;
+ short int valid;           // valid zeigt an, ob Shift korrekt ist
  char reserved [62];
 } DateShift;
 
@@ -244,7 +277,8 @@ typedef struct
  int startTime;
  int stopTime;
  char descr[768];
- char *comm;
+ // char *comm;
+ NotizControl *notiz;
  int reserverd [20];
  int dateShiftAnz;
  int dateShiftMax;
@@ -326,6 +360,7 @@ typedef struct
  UserGroupPerm gPerm[100];
  int gpanz;
  int calPerm;      // Rechte dritter am eigenen Kalender
+ // int defPerm;
  char defPermStr [64];
  char mail[4][128];
  char smtp[4][128];
@@ -351,8 +386,14 @@ typedef struct
  int notified4[256]; // -1 Terminierte Liste
  char color[8],color2[8];
  int selected;
+#ifdef WITH_THREADS
+ pthread_mutex_t connectMutex;
+ pthread_mutex_t reminderMutex;
+#endif
+ int connectionTime;
  int socket;
  int rsocket;  // socket for mrr
+ int ticket;  // ticket id of current/last request
  int direction_dateRemind;
  int direction_dateInvite;
  int direction_todoRemind;
@@ -366,6 +407,17 @@ typedef struct
  int internalStatus;
  char browser[64];
  char browserCmd[1024];
+ int  saveCalViewOnLogout;
+ int  savePrjViewOnLogout;
+ int  saveAdrViewOnLogout;
+ char *calView;
+ char *prjView;
+ char *adrView;
+ syncUserInfo *sync;
+ char *syncPref;
+ char *syncAdrPref;
+ char *syncPrjPref;
+ char *syncCalPref;
 } User;
 
 typedef struct
@@ -396,6 +448,70 @@ typedef struct
  int dontList; // == 1 wenn User nicht in User-Auflistung erscheinen soll.
 } Room;
 
+typedef struct {
+ User *user;
+ int  open;
+ int  type;
+ int  timeStamp;
+} syncControl;
+
+
+typedef struct {
+ int defaultFolder ;
+ char *defaultCategory;
+ int customFields[8];
+ int pdaPhonType;
+ int pdaAdrType;
+ int pdaAdrTypeHomeDone;
+ int adrAnz;
+ int adrNum;
+ int phonAnz;
+ int mobileAnz;
+ int faxAnz;
+ int mailAnz;
+ int wwwAnz;
+ int phonPrivAnz;
+ int mobilePrivAnz;
+ int faxPrivAnz;
+ int mailPrivAnz;
+ int wwwPrivAnz;
+
+ char *blockedCat[4096];
+ char *blockedCatStr;
+ int  *blockedCatLen;
+ int  blockedCatAnz;
+
+} adrSyncInfo;
+
+struct syncUserInfoStruct {
+ syncControl *ctrl;
+ int active;
+ int type;
+ int timeStamp;
+ int lastCalSync;
+ int lastPrjSync;
+ int lastAdrSync;
+ int mode;
+ // int formatType;
+ char *format;
+ int  startLine;
+ char *status;
+ char *status2;
+ int  csvImportFields[4096];
+ char  *csvImportTypeDefault[4096];
+ int  csvImportTypeField[4096];
+ int csvIds[4096];                  // csv-field-ids by type-tags
+ int syncFieldCount;
+ char csvSep;
+ csvData **csvTmp;
+ int csvTmpAnz;
+ int listPtr;
+ int tempPtr;
+ // parameter specific to one sync type
+ adrSyncInfo adr;
+ 
+ } ;
+
 
 
 #ifdef ___MAIN
@@ -404,6 +520,7 @@ typedef struct
 #define __EXTERN extern
 #endif
 
+__EXTERN syncControl syncCtrl;
 __EXTERN char *dataDir;
 __EXTERN User *userList;
 __EXTERN Group *groupList;
@@ -423,6 +540,8 @@ __EXTERN ReminderList reminderList;
 __EXTERN int nextReminder,lastReminderListCreation;
 
 
+__EXTERN char *serverDateID,*serverToDoID,*serverAddrID;
+
 __EXTERN char progName[64],company[128];
 
 __EXTERN int lastDateChange;
@@ -431,6 +550,7 @@ __EXTERN int today;
 __EXTERN int verbose;
 __EXTERN int noReminder;
 __EXTERN char *serverBinPath;
+__EXTERN char *clientBinPath;
 __EXTERN char *mail2smtp;
 
 __EXTERN char *mailMaskEinlExternText, *mailMaskEinlExternSubj;
@@ -443,6 +563,7 @@ __EXTERN char *mailMaskShiftInternText,*mailMaskShiftInternSubj;
 __EXTERN char *mailMaskSuspInternText ,*mailMaskSuspInternSubj;
 __EXTERN char *mailMaskShiftExternText,*mailMaskShiftExternSubj;
 __EXTERN char *mailMaskSuspExternText ,*mailMaskSuspExternSubj;
+__EXTERN char *serverMsg[1024];;
 
 
 __EXTERN char **holidays;
@@ -464,6 +585,8 @@ __EXTERN char *presenceList;
 
 //__EXTERN char serverLang[4];
 __EXTERN int serverLang;
+__EXTERN int enableBackup,permanentBackupHour;
+__EXTERN char *backupPath;
 
 void readLists();
 char *skip2nonWS(char *t);
@@ -477,6 +600,8 @@ char * uid2uname(int uid);
 char * uid2Fullname(int uid);
 int rname2rid(char *name);
 char * rid2rname(int rid);
+int name2FullID(char *name);
+
 void initDateList();
 int addDate(char *ds);
 void writeAllDates( );
@@ -495,8 +620,13 @@ int send2Client(int socket,char *msg);
 int split(char *txt,char **tp ,int max);
 int getUserPrefs (char *str,int socket);
 int setUserPrefs (char *str,int socket);
+int clearPWD (int socket,char *str );
 int saveUserPrefs (int uid);
-int  saveIDset (char *str,int socket);
+int sendUsersView(int socket, char *str);
+int saveUsersView(int socket,char *str);
+int getSyncPrefs(int socket, char *str);
+int saveSyncPrefs(int socket, char *str);
+int saveIDset (char *str,int socket);
 int getLists (int socket);
 int getUserList (int socket);
 int getGroupList (int socket);
@@ -543,6 +673,7 @@ int ignoreDate(int socket,char *str);
 void checkString(char *t);
 void clearUserSelection() ;
 void clearGroupSelection() ;
+int makeServerID( char *t);
 
 
 
@@ -589,3 +720,91 @@ int getPresence (int socket);
 int strChopLen(const char *t);
 int changeNotiz(int socket, char *str,int type);
 int sendNotizByMail(int socket, char *str,int type);
+
+
+
+//sync
+int getSyncParameters (int socket);
+int openSync( int socket, char *str);
+int initSync( int socket, char *str);
+int syncInto (int socket, char *str);
+int syncIntoStop (int socket, char *str);
+int getNextCsvFromServer(int socket, char *str);
+int endSync(  int socket, char *str);
+int closeSync(int socket, char *str);
+int closeSync(int uid);
+
+int syncInAdr (int socket, char *str);
+int readCSVfile(syncUserInfo *syncUI,char *file,char sep,int dirid, int start, int uid);
+int readCSVadrFormatFile(syncUserInfo *syncUI, char *file);
+int readCSVadrFormatString(syncUserInfo *syncUI, char *str);
+int addAdrCSVField(syncUserInfo *syncUI,char *field);
+char *endOfCsvEntry(char *t,char sep);
+int equivalentAddress(syncUserInfo *syncUI, Address *adr, int *diff);
+int syncAdrInto(int socket,syncUserInfo *syncUI,char *csv,int doImport);
+//int importCSV2Adr(syncUserInfo *syncUI, char *csv, int uid);
+csvData *  getnextAdrListCsv (int uid, syncUserInfo *syncUI);
+void fillCsvIntoAddress(syncUserInfo *syncUI,char *csv, Address *adr,int copyOnlyFull);
+int strEditDist(char *ostr, char *nstr, int maxDiff,int penalty);
+int strEditDist(char *ostr, char *nstr, int maxDiff);
+int compareAddress(Address *adrNew,Address *adrOld);
+
+
+
+void initSync ();
+char *cutLineFromDescr(syncUserInfo *syncUI,Address *adr, char *str);
+int tagInTaggedAdrText(char *str);
+int getSyncParameters (int socket);
+int stringIdentical(char *t1 , char *t2);
+int charIdentical(char t1 , char t2);
+
+
+
+
+NotizControl *initNotizList(int id,int type);
+NotizControl *_initNotizList(int id,int type);
+void clearNotizList ( NotizControl *nc);
+void extendNotizList(NotizControl *nc);
+void extendNotizList(int id, int type);
+
+
+int tid2idx(int tid);
+int pid2idx(int pid);
+int  permission4Proj(Project *proj, int uid);
+int  permission4Todo(TodoItem *todo, int uid);
+int readNotiz(Address *adr);
+int readNotiz(AddressFolder *adf);
+int readNotiz(TodoItem *todo);
+int readNotiz(Project *proj);
+int readNotiz(Date *proj);
+int readNotiz(FILE *fp,NotizControl *nc);
+int writeNotiz(FILE *fp, NotizControl *nc);
+int writeNotiz(Address *adr);
+int writeNotiz(AddressFolder *adf);
+int writeNotiz(TodoItem *todo);
+int writeNotiz(Project *proj);
+int writeNotiz(Date *proj);
+int nextNotizID(int id,int parent, int type);
+int _nextNotizID(NotizControl *nc, int parent);
+int addAdrNotiz(int aid,int parent, int uid,int date, int time, char *title, char *text);
+int addDirNotiz(int dirid,int parent, int uid,int date, int time, char *title, char *text);
+int addTaskNotiz(int tid,int parent, int uid,int date, int time, char *title, char *text);
+int addProjNotiz(int pid,int parent, int uid,int date, int time, char *title, char *text);
+int fillNotiz(Notiz *ntz,int parent, int uid,int date, int time, char *title, char *text);
+int changeTaskNotiz(int tid, int nid, char *text);
+int changeProjNotiz(int pid, int nid, char *text);
+int changeAddrNotiz(int aid, int nid, char *text);
+int changeDirNotiz(int dirid, int nid, char *text);
+int changeNotiz(NotizControl *nc, int nid, char *text);
+void * verifyNotizPointer(int pid, int type);
+
+
+//lang.cc
+void getLanguageDependedVariables ( );
+void getDeVariables ( );
+void getEnVariables ( );
+
+
+//utils.cc
+int nextExportTime(int hour, int min);
+void backupData();
