@@ -5,7 +5,7 @@
 //      Copyright (C) 2001-2002 by Rdiger Goetz
 //      Author: Rdiger Goetz <minkowsky@r-goetz.de>
 // 
-//      Time-stamp: <07-Jun-2002 09:12:19 goetz>
+//      Time-stamp: <22-Sep-2002 14:43:46 goetz>
 // 
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -306,8 +306,10 @@ void initUser(int id, char *name, char *full)
  userList[id].isNotified  = false;
  strcpy(userList[id].color,"#000000");
  strcpy(userList[id].color2,"#000000");
+ userList[id].connectionTime =0;
  userList[id].socket =-1;
  userList[id].rsocket =-1;
+ userList[id].ticket =-1;
  userList[id].direction_dateRemind =0;
  userList[id].direction_dateInvite =0;
  userList[id].direction_todoRemind =0;
@@ -323,8 +325,34 @@ void initUser(int id, char *name, char *full)
  
  initDateUserModification(&userList[id]);
  userList[id].dmodiDirty = false;
-}
+ 
+ userList[id].saveCalViewOnLogout=0;
+ userList[id].savePrjViewOnLogout=0;
+ userList[id].saveAdrViewOnLogout=0;
+ userList[id].calView = NULL;
+ userList[id].prjView = NULL;
+ userList[id].adrView = NULL;
 
+ userList[id].sync              = new syncUserInfo;
+ userList[id].sync->ctrl        = NULL;
+ userList[id].sync->active      = false;
+ userList[id].sync->timeStamp   = 0;
+ userList[id].sync->lastCalSync = 0;
+ userList[id].sync->lastPrjSync = 0;
+ userList[id].sync->lastAdrSync = 0;
+
+ userList[id].syncPref = NULL;
+ userList[id].syncAdrPref = NULL;
+ userList[id].syncPrjPref = NULL;
+ userList[id].syncCalPref = NULL;
+ for(int f=0;f<ADR_TAG_ANZ; f++)
+  userList[id].sync->csvImportTypeDefault[f] = NULL;
+
+#ifdef WITH_THREADS
+ pthread_mutex_init(&userList[id].connectMutex, NULL);
+ pthread_mutex_init(&userList[id].reminderMutex, NULL);
+#endif
+}
 void readUsers ()
 {
  FILE *fp;
@@ -1118,68 +1146,28 @@ char * rid2rname(int rid)
  return(unknownID);
 }
 
-char *skip2nonWS(char *t)
+int name2FullID(char *name)
 {
- while(*t==' ' || *t== '\t')
+ char *nam = name+2;
+ int id=-1;
+
+ if(name[0] == 'u')
   {
-   if(*t=='\n' || *t=='\0')
-    return(NULL);
-   t++;
+   if( (id = uname2uid(trimRight(nam))) <0)
+    return -1;
   }
- if(*t=='\n' || *t=='\0')
-  return(NULL);
- return(t);
-}
-char *skip2WS(char *t)
-{
- while(*t!=' ' && *t!= '\t')
+ else if(name[0] == 'g')
   {
-   if(*t=='\n' || *t=='\0')
-    return(NULL);
-   t++;
+   if( (id = gname2gid(trimRight(nam))) <0)
+    return -1;
+   id += GROUPOFFSET;
   }
- return(t);
+ else if(name[0] == 'r')
+  {
+   if( (id = rname2rid(trimRight(nam))) <0)
+    return -1;
+   id += ROOMOFFSET;
+  }
+ return id;
 }
 
-char *skip2nonWSnl(char *t)
-{
- while(*t==' ' || *t== '\t')
-  {
-   if( *t=='\0')
-    return(NULL);
-   t++;
-  }
- if( *t=='\0')
-  return(NULL);
- return(t);
-}
-char *skip2WSnl(char *t)
-{
- while(*t!=' ' && *t!= '\t')
-  {
-   if( *t=='\0')
-    return(NULL);
-   t++;
-  }
- return(t);
-}
-
-void chop(char *t)
-{
- char *s;
- s=t+strlen(t)-1;
- while(*s== '\n' || *s == '\r' || *s == '\0')
-  s--;
- s++;
- *s='\0';
-}
-char * strChop(char *t)
-{
- char *s;
- s=t+strlen(t)-1;
- while(*s== '\n' || *s == '\r' || *s == '\0')
-  s--;
- s++;
- *s='\0';
- return (t);
-}
